package com.softslate.commerce.businessobjects.core;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.EventBus;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.softslate.commerce.businessobjects.administrator.Administrator;
import com.softslate.commerce.businessobjects.events.EventBusRegistrar;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.daos.core.DAOFactory;

/**
 * An abstract useful for subclassing by classes that need to be invoked from
 * the command line. A method for initializing all of the system objects needed
 * to perform queries and invoke business objects is provided.
 * 
 * See the accompanying <code>CommandLineExample</code> for an example of a
 * subclass which may be invoked from the command line, and
 * <code>commandLineExample.sh</code> for a Unix shell script that may be used
 * to fire off the example.
 * 
 * @author dtobey
 * 
 */
public abstract class CommandLineSupport {

	static Log log = LogFactory.getLog(CommandLineSupport.class);

	public static Map initializeSystem(String adminUserName) {

		// Initialize settings from .properties files
		Properties appSettings;
		Properties appComponents;
		Properties appComponentsDefaults;
		Properties hibernate;
		try {
			appSettings = initAppSettings();
			appComponentsDefaults = initAppComponentsDefaults();
			appComponents = initAppComponents(appComponentsDefaults);
			hibernate = initHibernateSettings();
		} catch (Exception e) {
			log.error("Error loading settings from .properties files: ", e);
			return null;
		}

		// Initialize the DAOFactory
		DAOFactory daoFactory;
		try {
			daoFactory = initDAOFactory(appSettings, appComponents, appComponentsDefaults, hibernate);
		} catch (Exception e) {
			log.error("Couldn't initialize DAOFactory", e);
			return null;
		}

		// Create a BusinessObjectFactory.
		BusinessObjectFactory bof;
		try {
			bof = initBusinessObjectFactory(appSettings, appComponents, daoFactory);
		} catch (Exception e) {
			log.error("Error creating BusinessObjectFactory", e);
			return null;
		}

		// Load the Settings object from the database
		Settings settings = null;
		try {
			settings = bof.createObject(Settings.class);
			settings.initialize();
		} catch (Exception e) {
			log.error("Couldn't load settings", e);
			return null;
		}

		// Populate the factories with the database settings
		daoFactory.setSettings(settings);
		bof.setSettings(settings);

		// Initialize Guice
		if (settings != null && settings.getValue("initializeGuice") != null
				&& settings.getValue("initializeGuice").equals("1")) {
			GuiceBaseModule module = new GuiceBaseModule(appComponents, appComponentsDefaults, bof.getTransferMap(),
					true);
			GuiceCustomModule customModule = new GuiceCustomModule();
			Injector injector = Guice.createInjector(module, customModule);
			daoFactory.setInjector(injector);
			bof.setInjector(injector);
		}

		// Initialize EventBus
		initializeEventBus(bof);

		// Initialize a new user, including an Administrator object with a user
		// name for logging
		User user = null;
		try {
			user = bof.createObject(User.class);
			user.setLocale(Locale.getDefault());
			user.setAdministrator(bof.createObject((Administrator.class)));
			user.getAdministrator().setUserName(adminUserName);
			bof.setUser(user);
		} catch (Exception e) {
			if (log.isWarnEnabled())
				log.warn("Failed to initialize a User object.", e);
			return null;
		}

		// Put the system objects in a Map
		Map systemObjects = new HashMap();
		systemObjects.put("appSettings", appSettings);
		systemObjects.put("appComponents", appComponents);
		systemObjects.put("hibernate", hibernate);
		systemObjects.put("daoFactory", daoFactory);
		systemObjects.put("bof", bof);
		systemObjects.put("settings", settings);
		systemObjects.put("user", user);

		return systemObjects;

	}

	protected static Properties initAppSettings() throws IOException {
		Properties appSettings = new Properties();
		String appSettingsPropertiesFile = "appSettings.properties";
		if (System.getProperty("appSettings.configuration") != null) {
			appSettingsPropertiesFile = System.getProperty("appSettings.configuration");
		}
		appSettings.load(appSettings.getClass().getResourceAsStream("/" + appSettingsPropertiesFile));
		return appSettings;
	}

	protected static Properties initAppComponentsDefaults() throws IOException {
		Properties appComponentsDefaults = new Properties();
		Properties defaults = null;
		appComponentsDefaults = AppUtils.loadProperties("/appComponents.properties", defaults);
		return appComponentsDefaults;
	}

	protected static Properties initAppComponents(Properties appComponentsDefaults) throws IOException {
		Properties appComponents = new Properties();
		appComponents = AppUtils.loadProperties("/appComponents-custom.properties", appComponentsDefaults);
		return appComponents;
	}

	protected static Properties initHibernateSettings() throws IOException {
		Properties hibernate = new Properties();
		String hibernatePropertiesFile = "hibernate.properties";
		if (System.getProperty("hibernate.configuration") != null) {
			hibernatePropertiesFile = System.getProperty("hibernate.configuration");
		}

		// hibernate.configuration could represent a full path to a file outside
		// the classpath
		File propertiesFile = new File(hibernatePropertiesFile);
		if (hibernatePropertiesFile.startsWith("/") && propertiesFile.exists()) {
			hibernate = AppUtils.loadProperties(propertiesFile);
		} else {
			hibernate.load(hibernate.getClass().getResourceAsStream("/" + hibernatePropertiesFile));
		}
		return hibernate;
	}

	protected static DAOFactory initDAOFactory(Properties appSettings, Properties appComponents,
			Properties appComponentsDefaults, Properties hibernate) throws Exception {
		DAOFactory daoFactory = null;
		String className = appComponents.getProperty("daoFactoryImplementer");
		Class classDefinition = Class.forName(className);
		daoFactory = (DAOFactory) classDefinition.newInstance();
		daoFactory.setHibernateSettings(hibernate);
		daoFactory.setAppComponents(appComponents);
		daoFactory.setAppComponentsDefaults(appComponentsDefaults);
		daoFactory.setAppSettings(appSettings);
		daoFactory.initialize();
		return daoFactory;
	}

	protected static BusinessObjectFactory initBusinessObjectFactory(Properties appSettings, Properties appComponents,
			DAOFactory daoFactory) throws Exception {
		String className = appComponents.getProperty("businessObjectFactoryImplementer");
		Class classDefinition = Class.forName(className);
		BusinessObjectFactory bof = (BusinessObjectFactory) classDefinition.newInstance();
		bof.setAppSettings(appSettings);
		bof.setAppComponents(appComponents);
		bof.setDaoFactory(daoFactory);
		return bof;
	}

	protected static EventBus initializeEventBus(BusinessObjectFactory bof) {
		EventBus eventBus = new EventBus();
		EventBusRegistrar ebr;
		try {
			ebr = bof.createObject(EventBusRegistrar.class);
			ebr.initializeEventBus(new EventBus());
			bof.setEventBus(eventBus);
		} catch (Exception e) {
			log.error("Error initializing an EventBus");
		}
		return eventBus;
	}
}
