package com.softslate.commerce.businessobjects.customer;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.order.Order;

/**
 * Interface for business logic methods having to do with customer accounts and
 * related objects. Created in the Struts layer in response to requests for
 * information about an exisiting customer account, or to create new accounts.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CustomerProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "customerProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "customerProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.customer.BasicCustomerProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface CustomerProcessor extends BusinessProcessor {

	/**
	 * Processes a request to register or add a new customer account into the
	 * system. The incoming <code>Map</code> should have at least two keys:
	 * <code>userName</code> and <code>decryptedPassword</code>, which will be
	 * used for the new account. If successful, the <code>customer</code>
	 * property of the current <code>User</code> will be populated with the new
	 * account's information. If an account already exists with the same
	 * <code>userName</code>, the returned <code>Map</code> will contain a key
	 * named "resultCode" whose value is "errors.userAlreadyLoggedIn". This code
	 * is used in the Struts layer to look up the error message.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the user name and password for
	 *            the new customer account.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map processRegister(Map parameters) throws Exception;

	/**
	 * Processes a login attempt given a user name and password. The incoming
	 * <code>Map</code> must have two keys: <code>userName</code> and
	 * <code>password</code>. If the login attempt succeeds, the
	 * <code>Customer</code> property of the current <code>User</code> will be
	 * populated with the database information for the customer account. If the
	 * user name and password don't match an account in the database, the
	 * returned <code>Map</code> will contain a key named "resultCode" whose
	 * value is "errors.invalidLogin". This code is used in the Struts layer to
	 * look up the error message.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the user name and password for
	 *            the customer attempting to log in.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map processLogin(Map parameters) throws Exception;

	/**
	 * Processes an automatic login given a user's autoLoginToken. The incoming
	 * <code>Map</code> must have the key: <code>autoLoginToken</code>. If the
	 * login succeeds, the <code>Customer</code> property of the current
	 * <code>User</code> will be populated with the database information for the
	 * customer account. If the autoLoginToken doesn't match an account in the
	 * database, the returned <code>Map</code> will contain a key named
	 * "resultCode" whose value is "errors.invalidLogin".
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the autoLoginToken for the
	 *            customer.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map processAutoLogin(Map parameters) throws Exception;

	/**
	 * Updates a customer's address information. The incoming <code>Map</code>
	 * should contain keys corresponding to the delivery address properties of
	 * {@link com.softslate.commerce.businessobjects.customer.CustomerAddress},
	 * and the billing address properties of <code>Customer</code>. The
	 * <code>Customer</code> property of the current user, and the first
	 * <code>CustomerAddress</code> in the
	 * <code>customerAddressCollection</code> of the <code>Customer</code> are
	 * modified with the incoming address information. Then the database records
	 * for the customer and the customer address are updated in
	 * <code>sscCustomer</code> and <code>sscCustomerAddress</code>.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the address information for the
	 *            current user, which is to be used to update the account.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map updateAddresses(Map parameters) throws Exception;

	/**
	 * Updates a customer's account information. The incoming <code>Map</code>
	 * should contain keys corresponding to the properties of
	 * {@link com.softslate.commerce.businessobjects.customer.Customer}. The
	 * <code>Customer</code> property of the current user is modified with the
	 * account information contained in the incoming <code>Map</code>. Then the
	 * database record for the customer is updated. Used to process a changed
	 * user name and password.
	 * <p>
	 * If the incoming user name is already used by a different customer, the
	 * returned <code>Map</code> will contain a key named "resultCode" whose
	 * value is "errors.userNameNotUnique". This code is used in the Struts
	 * layer to look up the error message.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the account information for the
	 *            current user, which is to be used to update the account.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map updateCustomer(Map parameters) throws Exception;

	/**
	 * Loads the <code>Customer</code> property of this
	 * <code>CustomerProcesor</code> from the database given a <code>Map</code>
	 * containing the <code>userName</code>. Used in the lost password form,
	 * when a customer must be retrieved from database without a password.
	 * <p>
	 * If the application is set to use one-way encryption for customer
	 * passwords, this method will generate a random password and update the
	 * customer's record with it.
	 * <p>
	 * If the customer corresponding to the user name in the <code>Map</code>
	 * does not exist in the database, the returned <code>Map</code> will
	 * contain a key named "resultCode" whose value is
	 * "errors.userNameNotFound". This code is used in the Struts layer to look
	 * up the error message.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the user name of a given
	 *            customer.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map loadCustomerLostPassword(Map parameters) throws Exception;

	/**
	 * Loads a customer's order history from the database. If successful, the
	 * <code>Customer</code> property of the current <code>User</code> will have
	 * its <code>orderHistory</code> property populated with information from
	 * the database about all of the customer's previous orders in the store.
	 * 
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map loadOrderHistory() throws Exception;

	/**
	 * Loads all of the database information corresponding to a given order
	 * number for a given customer. Used to retrieve the order details for a
	 * customer viewing his or her order history.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing a key named
	 *            <code>orderNumber</code> with the order number of a given
	 *            order.
	 * @return An <code>Order</code> populated with information from the
	 *         database.
	 * @throws Exception
	 */
	abstract Order loadOrderDetails(Map parameters) throws Exception;

	/**
	 * Loads the set of download files that the customer has purchased from the
	 * database. The set of files is determined by querying the
	 * <code>sscProductSetting</code> table for download files associated with
	 * products that the customer has purchased previously.
	 * 
	 * @return A <code>Collection</code> of <code>Map</code>s, one for each file
	 *         available for download by the customer.
	 * @throws Exception
	 */
	abstract Collection loadDownloadFiles() throws Exception;

	/**
	 * Saves an <code>Order</code> in the database as a saved cart, with a
	 * status of "Saved", which a customer can pick up later.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing arguments, by default null.
	 * @return A <code>Map</code> with a key named "resultCode", equal to "0" if
	 *         the result is successful.
	 * @throws Exception
	 */
	abstract Map saveCart(Map parameters) throws Exception;

	/**
	 * Loads a customer's list of saved carts from the database. Saved carts are
	 * stored as incomplete orders in the <code>sscOrder</code> database table
	 * with a status of "Saved".
	 * 
	 * @return A <code>Collection</code> of <code>Order</code> objects
	 *         representing the customer's saved carts.
	 * @throws Exception
	 */
	abstract Collection loadSavedCarts() throws Exception;

	/**
	 * Loads a saved cart for a customer from the database, assigning it as the
	 * customer's current cart. Saved carts are stored as incomplete orders in
	 * the <code>sscOrder</code> database table with a status of "Saved".
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing a key named <code>orderID</code>
	 *            with the order id of a given order.
	 * @return A <code>Map</code> object containing the results of the
	 *         reprocessing of the order, or null, if the
	 *         <code>refreshSaved Carts</code> setting is false.
	 * @throws Exception
	 */
	abstract Map loadSavedCart(Map parameters) throws Exception;

	/**
	 * Removes a saved cart for a customer from the database, changing the order
	 * record to a status of "Saved".
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing a key named <code>orderID</code>
	 *            with the order id of a given order.
	 * @return null in this implementation.
	 * @throws Exception
	 */
	abstract Map removeSavedCart(Map parameters) throws Exception;

	/**
	 * Loads a previously placed order for a customer from the database,
	 * assigning it as the customer's current cart.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing a key named <code>orderID</code>
	 *            with the order id of a given order.
	 * @return A <code>Map</code> object containing the results of the
	 *         reprocessing of the order, or null, if the
	 *         <code>refreshReorders</code> setting is false
	 * @throws Exception
	 */
	abstract Map loadPreviousOrder(Map parameters) throws Exception;

	abstract Map loadCustomersAndCount(Map parameters) throws Exception;

	abstract Map addCustomer(Map parameters) throws Exception;

	abstract Map validateCustomer(Customer customer) throws Exception;

	abstract Customer loadCustomerFromID(Map parameters) throws Exception;

	abstract Map editCustomer(Map parameters) throws Exception;

	abstract Customer loadCustomerFromUserName(Map parameters) throws Exception;

	abstract Map editDeleteCustomers(Map parameters) throws Exception;

	abstract void deleteCustomer(Map parameters) throws Exception;

	abstract int loadMaxCustomerID() throws Exception;

	abstract Map loadCustomerAdvancedLostPassword(Map parameters) throws Exception;

	abstract Map loadCustomerFromResetToken(Map parameters) throws Exception;

	abstract Map resetPassword(String token, String newPassword) throws Exception;

	abstract Map processChangePassword(String currentDecryptedPassword, String newDecryptedPassword) throws Exception;

	abstract void customerChangePassword(String newDecryptedPassword, Customer customer) throws Exception;

	abstract boolean sendLostPasswordAdvancedEmail(Customer customer) throws Exception;

	abstract boolean sendCompleteResetPasswordEmail(Customer customer) throws Exception;

	abstract Map refreshCustomerFromDatabase(User user) throws Exception;

}
