package com.softslate.commerce.businessobjects.importexport;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.Country;
import com.softslate.commerce.businessobjects.order.State;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.Option;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.tax.BasicTaxRate;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.order.CountryDAO;
import com.softslate.commerce.daos.order.StateDAO;
import com.softslate.commerce.daos.product.AttributeDAO;
import com.softslate.commerce.daos.product.CategoryDAO;
import com.softslate.commerce.daos.product.ManufacturerDAO;
import com.softslate.commerce.daos.product.OptionDAO;
import com.softslate.commerce.daos.product.ProductDAO;
import com.softslate.commerce.daos.product.ProductGatewayDAO;
import com.softslate.commerce.daos.product.ProductSettingDAO;
import com.softslate.commerce.daos.product.SKUDAO;
import com.softslate.commerce.daos.tax.BasicTaxRateDAO;

/**
 * Business object class handling the processing of a text file import.
 * 
 * @author David Tobey, Ashish Jain
 */
public class BasicImportProcessor extends BasicImportExportProcessor implements ImportProcessor {

	static Log log = LogFactory.getLog(BasicImportProcessor.class);

	protected Collection resultMessages = new ArrayList();

	protected boolean insertBehavior = true;

	protected boolean updateBehavior = true;

	public static final int INSERT = 1;

	public static final int UPDATE = 2;

	public static final int SKIP = 3;

	public static final int FAIL = -1;

	protected ProductGatewayDAO productGatewayDAO = null;

	public Map processImport(String tableName, Collection businessObjects, Map parameters) throws Exception {
		Map resultParams = new HashMap();

		// Keep track of how many records to process at once
		int maxRecordsProcessed = 100;
		try {
			maxRecordsProcessed = Integer.parseInt((String) getSettings().getValue("maxRecordsProcessed"));
		} catch (Exception e) {
			log.warn("Error setting maxRecordsProcessed, defaulting to 100: ", e);
		}

		// Define the type of import - insert, update or both
		insertBehavior = parameters == null || parameters.get("importBehavior") == null
				|| parameters.get("importBehavior").equals("insertOnly")
				|| parameters.get("importBehavior").equals("insertAndUpdate");

		updateBehavior = parameters == null || parameters.get("importBehavior") == null
				|| parameters.get("importBehavior").equals("updateOnly")
				|| parameters.get("importBehavior").equals("insertAndUpdate");

		// Keep track of inserts, updates, failures
		int totalInserts = 0, totalFails = 0, totalUpdates = 0, totalSkipped = 0;
		boolean commitEachQuery = false;

		if (null != businessObjects) {
			Iterator iterator = businessObjects.iterator();
			while (iterator.hasNext()) {
				Map boMap = (Map) iterator.next();
				int resultCode = processImport(tableName, boMap, commitEachQuery);
				switch (resultCode) {
				case INSERT:
					totalInserts++;
					break;
				case UPDATE:
					totalUpdates++;
					break;
				case SKIP:
					totalSkipped++;
					break;
				case FAIL:
					totalFails++;
					break;
				}

				// Clear the Hibernate session to preserve memory
				if (totalInserts % maxRecordsProcessed == 0 || totalUpdates % maxRecordsProcessed == 0) {
					getDaoFactory().getSession().flush();
					getDaoFactory().getSession().clear();
					getDaoFactory().evictCache();
				}
			}

			if (!commitEachQuery)
				getDaoFactory().commitTransaction();
		}
		resultParams.put("insertRecordsCount", new Integer(totalInserts));
		resultParams.put("updateRecordsCount", new Integer(totalUpdates));
		resultParams.put("skippedRecordsCount", new Integer(totalSkipped));
		resultParams.put("failRecordsCount", new Integer(totalFails));
		resultParams.put("resultMessages", resultMessages);
		if (log.isInfoEnabled())
			log.info("Import results: " + resultParams);
		return resultParams;
	}

	public int processImport(String tableName, Map boMap, boolean commit) throws Exception {
		int returnCode = -1;
		if (null != tableName) {
			if (tableName.equalsIgnoreCase(PRODUCT_TABLE))
				returnCode = this.processProductImport(boMap, commit);
			else if (tableName.equalsIgnoreCase(CATEGORY_TABLE))
				returnCode = this.processCategoryImport(boMap, commit);
			else if (tableName.equalsIgnoreCase(ATTRIBUTE_TABLE))
				returnCode = this.processAttributeImport(boMap, commit);
			else if (tableName.equalsIgnoreCase(OPTION_TABLE))
				returnCode = this.processOptionImport(boMap, commit);
			else if (tableName.equalsIgnoreCase(SKU_TABLE))
				returnCode = this.processSKUImport(boMap, commit);
			else if (tableName.equalsIgnoreCase(PRODUCT_SETTING_TABLE))
				returnCode = this.processProductSettingImport(boMap, commit);
			else if (tableName.equalsIgnoreCase(BASICTAXRATE_TABLE))
				returnCode = this.processBasicTaxRateImport(boMap, commit);
			else
				throw new Exception("This table transformation not supported, Tablename : " + tableName);
		}
		return returnCode;
	}

	public int processImport(DAOInterface dao, Map objectMap, Object bean, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing import.");

		try {
			Object loadedObj = dao.loadObject(bean);
			boolean isInsert = (null == loadedObj) ? true : false;

			if (insertBehavior && isInsert) {

				// Special for SKUs: don't insert if a parent product was not
				// found. Attribute-only SKUs may not be inserted (though they
				// can be updated).
				if (bean instanceof SKU && BeanUtils.getProperty(bean, "product") == null) {
					return SKIP;
				}

				// Go ahead and insert
				dao.insertObject(bean, commit);
				return INSERT;
			} else if (updateBehavior && !isInsert) {

				// Update the found object
				BeanUtils.copyProperties(loadedObj, objectMap);
				dao.updateObject(loadedObj, commit);
				return UPDATE;
			} else {

				// The import behavior doesn't match what needs to be done
				return SKIP;
			}
		} catch (Exception ex) {
			log.error(ex.toString(), ex);
			resultMessages.add("Failure message: " + ex.toString());
			return FAIL;
		}
	}

	public int processProductImport(Map productMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing product import for Code = " + productMap.get("code"));
		if (log.isDebugEnabled())
			log.debug("productMap" + productMap);

		// Skip if the code doesn't exist
		if (productMap.get("code") == null || productMap.get("code").equals("")) {
			return FAIL;
		}

		// Set the SEO code to null if it's empty
		if (productMap.get("seoCode") != null && productMap.get("seoCode").equals("")) {
			productMap.put("seoCode", null);
		}

		// Create the product bean to insert or update
		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		if (null == this.productDAO)
			productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		Product bean =  bof.createObject(Product.class);
		BeanUtils.copyProperties(bean, productMap);

		// Assign the primary category, if it's there
		String primaryCategoryCode = (String) productMap.get("primaryCategoryCode");
		if (primaryCategoryCode != null && primaryCategoryCode.length() > 0) {
			if (null == this.categoryDAO)
				categoryDAO =  getDaoFactory().createDAO(CategoryDAO.class);
			Category category =  bof.createObject(Category.class);
			category.setCode(primaryCategoryCode);
			categoryDAO.loadObject(category);
			category = categoryDAO.getCategory();
			if (category != null) {
				bean.setPrimaryCategory(category);
				bean.setPrimaryCategoryID(new Integer(category.getCategoryID()));
				productMap.put("primaryCategory", category);
				productMap.put("primaryCategoryID", new Integer(category.getCategoryID()));
			} else {
				String resultMessage = "Warning message: While processing product '" + productMap.get("code")
						+ "', the importer could not find the referenced primary category '" + primaryCategoryCode
						+ "'. The other fields will be updated/inserted but not the " + "primary category.";
				resultMessages.add(resultMessage);
			}
		}

		// Assign the manufacturer, if it's there
		String manufacturerCode = (String) productMap.get("manufacturerCode");
		if (manufacturerCode != null && manufacturerCode.length() > 0) {
			if (null == this.manufacturerDAO)
				manufacturerDAO =  getDaoFactory().createDAO(ManufacturerDAO.class);
			Manufacturer manufacturer =  bof.createObject(Manufacturer.class);
			manufacturer.setCode(manufacturerCode);
			manufacturerDAO.loadObject(manufacturer);
			manufacturer = manufacturerDAO.getManufacturer();
			if (manufacturer != null) {
				bean.setManufacturer(manufacturer);
				bean.setManufacturerID(new Integer(manufacturer.getManufacturerID()));
				productMap.put("manufacturer", manufacturer);
				productMap.put("manufacturerID", new Integer(manufacturer.getManufacturerID()));
			} else {
				String resultMessage = "Warning message: While processing product '" + productMap.get("code")
						+ "', the importer could not find the referenced manufacturer '" + manufacturerCode
						+ "'. The other fields will be updated/inserted but not " + "the manufacturer.";
				resultMessages.add(resultMessage);
			}
		}

		// Run the import in the database
		int result = this.processImport(productDAO, productMap, bean, commit);
		if (result == SKIP && !insertBehavior && updateBehavior) {
			String resultMessage = "Skipped message: The record with code '" + productMap.get("code")
					+ "' was skipped because a matching record was not found and the "
					+ "Import Behavior is set to Update Only.";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && !updateBehavior && insertBehavior) {
			String resultMessage = "Skipped message: The record with code '" + productMap.get("code")
					+ "' was skipped because a matching record was found and the "
					+ "Import Behavior is set to Insert Only.";
			resultMessages.add(resultMessage);
		}
		
		// Update the categories if they exist
		Set<String> categories = new HashSet<String>();
		boolean hasExtraCatHeaders = false;
		for (int i = 0; i < 5; i++) {
			if (productMap.containsKey("category" + (i + 1))){
				hasExtraCatHeaders = true;
				String categoryCode = (String) productMap.get("category" + (i + 1));
				if (StringUtils.isNotBlank(categoryCode)) {
					categories.add(categoryCode);
				}
			}
		}
		// Run if we have any category1... headers so it deletes out
		// associations even if the new row doesn't have any new ones.
		if (hasExtraCatHeaders) {
			try {
				int added = updateExtraCategories((String) productMap.get("code"), categories);
				if (log.isInfoEnabled()) {
					log.info("Added associations for productCode: " + productMap.get("code") + " Qty: " + added);
				}
			} catch (Exception ex) {
				String message = "Exception when trying to update category associations for product: "
						+ productMap.get("code") + " ex: " + ex.getMessage();
				resultMessages.add(message);
				if (log.isErrorEnabled()) {
					log.error(message, ex);
				}
			}
		}
		return result;
	}

	protected int updateExtraCategories(String productCode, Set<String> categoryCodes) throws Exception {
		int added = 0;
		if (StringUtils.isNotBlank(productCode)) {
			if (productGatewayDAO == null) {
				productGatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
			}
			try {
				if (log.isInfoEnabled()) {
					log.info("Removing category associations for product: " + productCode);
				}
				int deleted = productGatewayDAO.removeCategoryAssociations(productCode);
				if (log.isInfoEnabled()) {
					log.info("Result: " + deleted);
				}
			} catch (Exception ex) {
				if (log.isInfoEnabled()) {
					log.info("Error removing category associations.", ex);
				}
				return 0;
			}
			if (categoryCodes != null && categoryCodes.size() > 0) {
				for (String categoryCode : categoryCodes) {
					if (StringUtils.isNotBlank(categoryCode)) {
						try {
							if (log.isInfoEnabled()) {
								log.info("Adding category association: " + categoryCode);
							}
							int count = productGatewayDAO.addCategoryAssociation(productCode, categoryCode);
							if (log.isInfoEnabled()) {
								log.info("Result: " + count);
							}
							added = added + count;
						} catch (Exception ex) {
							if (log.isInfoEnabled()) {
								log.info("Error adding category association for code: " + categoryCode, ex);
							}
						}
					}
				}
			}
		}
		return added;
	}

	public int processOptionImport(Map optionMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing Option import for optionCode = " + optionMap.get("optionCode")
					+ " and attributeCode = " + optionMap.get("attributeCode"));

		// Skip if the code doesn't exist
		if (optionMap.get("optionCode") == null || optionMap.get("optionCode").equals("")
				|| optionMap.get("attributeCode") == null || optionMap.get("attributeCode").equals("")) {
			return FAIL;
		}
		if (null == this.optionDAO)
			optionDAO =  getDaoFactory().createDAO(OptionDAO.class);

		// Code to get attributeID using attributeCode from option.getAttribute
		// object.
		if (null == this.attributeDAO)
			attributeDAO =  getDaoFactory().createDAO(AttributeDAO.class);
		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		Attribute attribute =  bof.createObject(Attribute.class);
		attribute.setAttributeCode((String) optionMap.get("attributeCode"));
		attributeDAO.loadObject(attribute);
		attribute = attributeDAO.getAttribute();
		if (null != attribute) {
			Option bean =  bof.createObject(Option.class);
			BeanUtils.copyProperties(bean, optionMap);
			bean.setAttributeID(attribute.getAttributeID());
			bean.setAttribute(attribute);

			// Run the import in the database
			int result = this.processImport(optionDAO, optionMap, bean, commit);
			if (result == SKIP && !insertBehavior && updateBehavior) {
				String resultMessage = "Skipped message: The record with code '" + optionMap.get("optionCode")
						+ "' was skipped because a matching record was not found and the "
						+ "Import Behavior is set to Update Only.";
				resultMessages.add(resultMessage);
			} else if (result == SKIP && !updateBehavior && insertBehavior) {
				String resultMessage = "Skipped message: The record with code '" + optionMap.get("optionCode")
						+ "' was skipped because a matching record was found and the "
						+ "Import Behavior is set to Insert Only.";
				resultMessages.add(resultMessage);
			}
			return result;
		} else {
			String resultMessage = "Failure message: While processing option '" + optionMap.get("optionCode")
					+ "', the importer could not find the referenced attribute '" + optionMap.get("attributeCode");
			resultMessages.add(resultMessage);
			return FAIL;
		}

	}

	public int processAttributeImport(Map attributeMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing Attribute import for Code = " + attributeMap.get("attributeCode"));

		// Skip if the code doesn't exist
		if (attributeMap.get("attributeCode") == null || attributeMap.get("attributeCode").equals("")) {
			return FAIL;
		}
		if (null == this.attributeDAO)
			attributeDAO =  getDaoFactory().createDAO(AttributeDAO.class);

		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		Attribute bean =  bof.createObject(Attribute.class);
		BeanUtils.copyProperties(bean, attributeMap);

		// Run the import in the database
		int result = this.processImport(attributeDAO, attributeMap, bean, commit);
		if (result == SKIP && !insertBehavior && updateBehavior) {
			String resultMessage = "Skipped message: The record with code '" + attributeMap.get("attributeCode")
					+ "' was skipped because a matching record was not found and the "
					+ "Import Behavior is set to Update Only.";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && !updateBehavior && insertBehavior) {
			String resultMessage = "Skipped message: The record with code '" + attributeMap.get("attributeCode")
					+ "' was skipped because a matching record was found and the "
					+ "Import Behavior is set to Insert Only.";
			resultMessages.add(resultMessage);
		}
		return result;
	}

	public int processSKUImport(Map skuMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing SKU import for SKU Number = " + skuMap.get("skuNumber"));

		// Skip if the number doesn't exist
		if (skuMap.get("skuNumber") == null || skuMap.get("skuNumber").equals("")) {
			return FAIL;
		}

		// Chop off decimals for the inventoryLevel
		if (skuMap.get("inventoryLevel") != null) {
			String inventoryLevel = (String) skuMap.get("inventoryLevel");
			if (inventoryLevel.endsWith(".00")) {
				inventoryLevel = inventoryLevel.substring(0, inventoryLevel.length() - 3);
				skuMap.put("inventoryLevel", inventoryLevel);
			}
		}

		if (null == this.skuDAO)
			skuDAO =  getDaoFactory().createDAO(SKUDAO.class);
		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		SKU bean =  bof.createObject(SKU.class);
		BeanUtils.copyProperties(bean, skuMap);

		// Assign the product, if it's there
		String productCode = (String) skuMap.get("productCode");
		if (productCode != null && productCode.length() > 0) {
			if (null == this.productDAO)
				productDAO =  getDaoFactory().createDAO(ProductDAO.class);
			Product product =  bof.createObject(Product.class);
			product.setCode(productCode);
			productDAO.loadObject(product);
			product = productDAO.getProduct();
			if (product != null) {
				bean.setProduct(product);
				bean.setProductID(new Integer(product.getProductID()));
				skuMap.put("product", product);
				skuMap.put("productID", new Integer(product.getProductID()));
			} else {
				String resultMessage = "Warning message: While processing sku '" + skuMap.get("skuNumber")
						+ "', the importer could not find the referenced product '" + productCode
						+ "'. The importer will NOT insert this record. " + "If the import behavior allows for "
						+ "updates and a matching record is found, the importer "
						+ "will attempt to update the other fields, " + "but not the product.";
				resultMessages.add(resultMessage);
			}
		}

		// Run the import in the database
		int result = this.processImport(skuDAO, skuMap, bean, commit);
		if (result == SKIP && !insertBehavior && updateBehavior) {
			String resultMessage = "Skipped message: The record with SKU number '" + skuMap.get("skuNumber")
					+ "' was skipped because a matching record was not found and the "
					+ "Import Behavior is set to Update Only.";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && !updateBehavior && insertBehavior && bean.getProduct() == null) {
			String resultMessage = "Skipped message: The record with SKU number '" + skuMap.get("skuNumber")
					+ "' was skipped because no parent product could"
					+ "not be found. (Attribute-only SKUs may not be inserted " + "through the importer.)";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && !updateBehavior && insertBehavior) {
			String resultMessage = "Skipped message: The record with SKU number '" + skuMap.get("skuNumber")
					+ "' was skipped because a matching record was found and the "
					+ "Import Behavior is set to Insert Only.";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && bean.getProduct() == null) {
			String resultMessage = "Skipped message: The record with SKU number '" + skuMap.get("skuNumber")
					+ "' was skipped because no parent product could "
					+ "not be found. (Attribute-only SKUs may not be inserted " + "through the importer.)";
			resultMessages.add(resultMessage);
		}
		return result;
	}

	public int processCategoryImport(Map categoryMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing Category import for Code = " + categoryMap.get("code"));

		// Skip if the code doesn't exist
		if (categoryMap.get("code") == null || categoryMap.get("code").equals("")) {
			return FAIL;
		}
		if (null == this.categoryDAO)
			categoryDAO =  getDaoFactory().createDAO(CategoryDAO.class);
		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		Category bean =  bof.createObject(Category.class);
		BeanUtils.copyProperties(bean, categoryMap);

		// Assign the parent category, if it's there
		String parentCategoryCode = (String) categoryMap.get("parentCategoryCode");
		if (log.isDebugEnabled())
			log.debug("Looking up parent category: " + parentCategoryCode);
		if (parentCategoryCode != null && parentCategoryCode.length() > 0) {
			if (bean.getCode().equals(parentCategoryCode)) {
				String resultMessage = "Warning message: A category may not be " + "a parent of itself. '"
						+ categoryMap.get("code") + "', identifies itself as its own parent. The other fields "
						+ "will be updated/inserted but not the parent category.";
				resultMessages.add(resultMessage);
			} else {
				Category category =  bof.createObject(Category.class);
				category.setCode(parentCategoryCode);
				CategoryDAO newDAO =  getDaoFactory().createDAO(CategoryDAO.class);
				newDAO.loadObject(category);
				category = newDAO.getCategory();
				if (category != null) {
					if (log.isDebugEnabled())
						log.debug("Found parent category: " + category.getCode());
					bean.setParent(category);
					bean.setParentCategory(new Integer(category.getCategoryID()));
					categoryMap.put("parent", category);
					categoryMap.put("parentCategory", new Integer(category.getCategoryID()));
				} else {
					String resultMessage = "Warning message: While processing category '" + categoryMap.get("code")
							+ "', the importer could not find the referenced parent " + "category '"
							+ parentCategoryCode + "'. The other fields will be updated/inserted but not the "
							+ "parent category.";
					resultMessages.add(resultMessage);
				}
			}
		}

		// Run the import in the database
		int result = this.processImport(categoryDAO, categoryMap, bean, commit);
		if (result == SKIP && !insertBehavior && updateBehavior) {
			String resultMessage = "Skipped message: The record with code '" + categoryMap.get("code")
					+ "' was skipped because a matching record was not found and the "
					+ "Import Behavior is set to Update Only.";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && !updateBehavior && insertBehavior) {
			String resultMessage = "Skipped message: The record with code '" + categoryMap.get("code")
					+ "' was skipped because a matching record was found and the "
					+ "Import Behavior is set to Insert Only.";
			resultMessages.add(resultMessage);
		}
		return result;
	}

	public int processProductSettingImport(Map productSettingMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing product setting import for Code = " + productSettingMap.get("code"));
		if (log.isDebugEnabled())
			log.debug("productSettingMap" + productSettingMap);

		// Skip if the code doesn't exist
		if (productSettingMap.get("code") == null || productSettingMap.get("code").equals("")
				|| productSettingMap.get("productCode") == null || productSettingMap.get("productCode").equals("")) {
			return FAIL;
		}

		// Code to get productID using productCode
		if (null == this.productDAO)
			productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		Product product =  bof.createObject(Product.class);
		product.setCode((String) productSettingMap.get("productCode"));
		productDAO.loadObject(product);
		product = productDAO.getProduct();
		if (null != product) {

			// Create the product setting bean to insert or update
			if (null == this.productSettingDAO)
				productSettingDAO =  getDaoFactory().createDAO(ProductSettingDAO.class);
			ProductSetting bean =  bof.createObject(ProductSetting.class);
			BeanUtils.copyProperties(bean, productSettingMap);
			bean.setProductID(product.getProductID());
			bean.setProduct(product);

			// Run the import in the database
			int result = this.processImport(productSettingDAO, productSettingMap, bean, commit);
			if (result == SKIP && !insertBehavior && updateBehavior) {
				String resultMessage = "Skipped message: The record with code '" + productSettingMap.get("code")
						+ " and product code '" + productSettingMap.get("productCode")
						+ "' was skipped because a matching record was not found and the "
						+ "Import Behavior is set to Update Only.";
				resultMessages.add(resultMessage);
			} else if (result == SKIP && !updateBehavior && insertBehavior) {
				String resultMessage = "Skipped message: The record with code '" + productSettingMap.get("code")
						+ " and product code '" + productSettingMap.get("productCode")
						+ "' was skipped because a matching record was found and the "
						+ "Import Behavior is set to Insert Only.";
				resultMessages.add(resultMessage);
			}
			return result;
		} else {
			String resultMessage = "Failure message: While processing product setting '"
					+ productSettingMap.get("code") + "', the importer could not find the referenced product '"
					+ productSettingMap.get("productCode") + "'";
			resultMessages.add(resultMessage);
			return FAIL;
		}
	}

	public int processBasicTaxRateImport(Map basicTaxRateMap, boolean commit) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing BasicTaxRate import.");
		if (null == this.basicTaxRateDAO)
			basicTaxRateDAO = getDaoFactory().createDAO(BasicTaxRateDAO.class);
		BusinessObjectFactory bof = this.getBusinessObjectFactory();
		BasicTaxRate bean = bof.createObject(BasicTaxRate.class);
		BeanUtils.copyProperties(bean, basicTaxRateMap);

		// Assign the state, if it's there
		String stateCode = (String) basicTaxRateMap.get("stateCode");
		State state = null;
		if (stateCode != null && stateCode.length() > 0) {
			if (null == this.stateDAO)
				stateDAO = getDaoFactory().createDAO(StateDAO.class);
			state = bof.createObject(State.class);
			state.setCode(stateCode);
			stateDAO.loadObject(state);
			state = stateDAO.getState();
			if (state != null) {
				bean.setState(state);
				bean.setStateID(new Integer(state.getStateID()));
				basicTaxRateMap.put("state", state);
				basicTaxRateMap.put("stateID", new Integer(state.getStateID()));
			} else {
				String resultMessage = "Warning message: While processing a BasicTaxRate, "
						+ "the importer could not find the referenced state '" + stateCode
						+ "'. The importer will NOT insert this record. " + "If the import behavior allows for "
						+ "updates and a matching record is found, the importer "
						+ "will attempt to update the other fields, " + "but not the state.";
				resultMessages.add(resultMessage);
			}
		}

		// Assign the country, if it's there
		String countryCode = (String) basicTaxRateMap.get("countryCode");
		Country country = null;
		if (countryCode != null && countryCode.length() > 0) {
			if (null == this.countryDAO)
				countryDAO = getDaoFactory().createDAO(CountryDAO.class);
			country = bof.createObject(Country.class);
			country.setCode(countryCode);
			countryDAO.loadObject(country);
			country = countryDAO.getCountry();
			if (country != null) {
				bean.setCountry(country);
				bean.setCountryID(new Integer(country.getCountryID()));
				basicTaxRateMap.put("country", country);
				basicTaxRateMap.put("countryID", new Integer(country.getCountryID()));
			} else {
				String resultMessage = "Warning message: While processing a BasicTaxRate, "
						+ "the importer could not find the referenced country '" + countryCode
						+ "'. The importer will NOT insert this record. " + "If the import behavior allows for "
						+ "updates and a matching record is found, the importer "
						+ "will attempt to update the other fields, " + "but not the country.";
				resultMessages.add(resultMessage);
			}
		}

		// Validate that the state and country match
		if (state != null && country != null && state.getCountryID() != country.getCountryID()) {
			String resultMessage = "Skipped message: The record with basicTaxRateID '"
					+ basicTaxRateMap.get("basicTaxRateID")
					+ "' was skipped because the submitted stateCode and countryCode do not match";
			resultMessages.add(resultMessage);
			return SKIP;
		} 
		
		// If a valid state was submitted and no country, fill in the country with the state's country
		else if (state != null && country == null) {
			bean.setCountry(state.getCountry());
			bean.setCountryID(new Integer(state.getCountryID()));
			basicTaxRateMap.put("country", state.getCountry());
			basicTaxRateMap.put("countryID", new Integer(state.getCountryID()));
		}

		// Run the import in the database
		int result = this.processImport(basicTaxRateDAO, basicTaxRateMap, bean, commit);
		if (result == SKIP && !insertBehavior && updateBehavior) {
			String resultMessage = "Skipped message: The record with basicTaxRateID '"
					+ basicTaxRateMap.get("basicTaxRateID")
					+ "' was skipped because a matching record was not found and the "
					+ "Import Behavior is set to Update Only.";
			resultMessages.add(resultMessage);
		} else if (result == SKIP && !updateBehavior && insertBehavior) {
			String resultMessage = "Skipped message: The record with basicTaxRateID '"
					+ basicTaxRateMap.get("basicTaxRateID")
					+ "' was skipped because a matching record was found and the "
					+ "Import Behavior is set to Insert Only.";
			resultMessages.add(resultMessage);
		}
		return result;

	}

}
