package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import paypal.payflow.*;
import com.softslate.commerce.businessobjects.payment.paypal.PayPalConstants;
import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * 
 * Base class for all payment processors that use PayPal's name-value pair (NVP)
 * API to process payments. Common functionality as been distilled to this class
 * for each of the processors, including PayPal UK, PayPal ACH, and Payflow Pro.
 * 
 * @author Ashish Jain, David Tobey
 * 
 */
public abstract class BasePayPalNVPProcessor extends BasePaymentProcessor {
	static Log log = LogFactory.getLog(BasePayPalNVPProcessor.class);

	public static final String HOST_ADDRESS = "HostAddress";

	public static final String HOST_PORT = "HostPort";

	public static final String TIME_OUT = "Timeout";

	public static final String PROXY_ADDR = "ProxyAddress";

	public static final String PROXY_PORT = "ProxyPort";

	public static final String PROXY_LOGON = "ProxyLogon";

	public static final String PROXY_PASSWD = "ProxyPassword";

	public static final String CERT_PATH = "certPath";

	public static final String PARTNER = "Partner";

	public static final String VENDOR = "Vendor";

	public static final String USER = "User";

	public static final String PASSWORD = "Password";

	public static final String COMMENT1 = "Comment1";

	public static final String COMMENT2 = "Comment2";

	public static final String CUSTOMER_REF = "CustRef";

	public static final String PO_NUMBER = "PONumber";

	public static final String CUSTOMER_CODE = "CustomerCode";

	public static final String DESCRIPTION = "Desc";

	public static final String VERBOSITY = "Verbosity";

	protected abstract void setPaymentVariablesToPersist(Payment payment,
			Map parameters, Map responseMap);

	// For payflowPro, keyPrefix = payflowPro
	// For paypal website pro UK , keyPrefix = paypalUK
	public String makePayPalCall(Map parameters, String keyPrefix)
			throws Exception {
		PayflowAPI payflowProConnection = initializeConnection(keyPrefix);
		if (log.isInfoEnabled())
			log.info("PFProAPI Version : " + payflowProConnection.getVersion());
		Map params = this.preparePaypalRequest(parameters, keyPrefix);
		String parmList = this.prepareRequestData(params, false);
		String requestID = getUser().getOrder().getOrderID()
				+ PayflowUtility.getRequestId();
		String response = null;
		response = payflowProConnection.submitTransaction(parmList, requestID);
		return response;
	}

	public PayflowAPI initializeConnection(String keyPrefix) throws Exception {
		String hostAddress = (String) getSettings().getValue(
				keyPrefix + HOST_ADDRESS);
		String hostPort = (String) getSettings()
				.getValue(keyPrefix + HOST_PORT);
		String timeout = (String) getSettings().getValue(keyPrefix + TIME_OUT);
		String proxyAddress = (String) getSettings().getValue(
				keyPrefix + PROXY_ADDR);
		String proxyPort = (String) getSettings().getValue(
				keyPrefix + PROXY_PORT);
		String proxyLogon = (String) getSettings().getValue(
				keyPrefix + PROXY_LOGON);
		String proxyPasswd = (String) getSettings().getValue(
				keyPrefix + PROXY_PASSWD);
		int hostPortInt = 0;
		try {
			hostPortInt = Integer.parseInt(hostPort);
		} catch (NumberFormatException e) {
			log.debug("HostPort isn't numeric", e);
		}
		int timeoutInt = 0;
		try {
			timeoutInt = Integer.parseInt(timeout);
		} catch (NumberFormatException e) {
			log.debug("Timeout isn't numeric", e);
		}
		int proxyPortInt = 0;
		try {
			proxyPortInt = Integer.parseInt(proxyPort);
		} catch (NumberFormatException e) {
			log.debug("Proxy Port isn't numeric", e);
		}
		return new PayflowAPI(hostAddress, hostPortInt, timeoutInt,
				proxyAddress, proxyPortInt, proxyLogon, proxyPasswd);
	}

	public Map preparePaypalRequest(Map parameters, String keyPrefix)
			throws Exception {
		Map params = new HashMap();
		params = this.preparePayPalAuthRequest(params, keyPrefix);
		params = this.preparePaypalRequestCommonPayload(params, parameters);
		params = this.preparePaypalRequestAcutalPayload(params, parameters);
		params = this.preparePaypalLengthParameters(params, parameters);
		return params;
	}

	public Map preparePayPalAuthRequest(Map params, String keyPrefix)
			throws Exception {
		if (null == params)
			params = new HashMap();
		params.put(PayPalConstants.PARTNER, getSettings().getValue(
				keyPrefix + PARTNER));
		String payflowProVendor = (String) getSettings().getValue(
				keyPrefix + VENDOR);
		if (payflowProVendor != null && payflowProVendor.length() > 0)
			params.put(PayPalConstants.VENDOR, payflowProVendor);
		params.put(PayPalConstants.USER, getSettings().getValue(
				keyPrefix + USER));
		params.put(PayPalConstants.PWD, getSettings().getValue(
				keyPrefix + PASSWORD));
		String verbosity = (String) getSettings().getValue(
				keyPrefix + VERBOSITY);
		if (verbosity != null && verbosity.length() > 0)
			params.put(PayPalConstants.VERBOSITY, verbosity);
		return params;
	}

	public Map preparePaypalRequestCommonPayload(Map params, Map parameters)
			throws Exception {
		if (null == params)
			params = new HashMap();
		if (parameters.get(PayPalConstants.TENDER) != null)
			params.put(PayPalConstants.TENDER, parameters
					.get(PayPalConstants.TENDER));
		if (parameters.get(PayPalConstants.TRXTYPE) != null)
			params.put(PayPalConstants.TRXTYPE, parameters
					.get(PayPalConstants.TRXTYPE));
		String totalStr = null;
		if (null != getUser().getOrder().getTotal())
			totalStr = getUser().getOrder().getTotal().toString();
		else
			totalStr = getUser().getOrder().getSubtotal().toString();
		params.put(PayPalConstants.AMT, totalStr);
		if (parameters.get(PayPalConstants.PNREF) != null)
			params.put(PayPalConstants.ORIGID, parameters
					.get(PayPalConstants.PNREF));
		if (parameters.get("comment1") != null)
			params.put(PayPalConstants.COMMENT1, parameters.get("comment1"));
		if (parameters.get("comment2") != null)
			params.put(PayPalConstants.COMMENT2, parameters.get("comment2"));
		if (parameters.get("description") != null)
			params.put(PayPalConstants.DESC, parameters.get("description"));
		if (parameters.get("custRef") != null)
			params.put(PayPalConstants.CUSTREF, parameters.get("custRef"));
		if (parameters.get("poNumber") != null)
			params.put(PayPalConstants.PONUM, parameters.get("poNumber"));
		if (parameters.get("customerCode") != null)
			params.put(PayPalConstants.PONUM, parameters.get("customerCode"));
		return params;
	}

	protected abstract Map preparePaypalRequestAcutalPayload(Map params,
			Map parameters);

	public Map preparePaypalLengthParameters(Map params, Map parameters) {
		Map newParams = new HashMap();
		Iterator i = params.keySet().iterator();
		while (i.hasNext()) {
			String key = (String) i.next();
			String value = (String) params.get(key);
			String keyWithLength = key + "[" + value.length() + "]";
			newParams.put(keyWithLength, value);
		}
		return newParams;
	}

	public void processSuccessfulTransaction(Map parameters, Map responseMap)
			throws Exception {
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		BeanUtils.copyProperties(payment, parameters);
		payment.setStatus("Received");
		payment.setAmount(getUser().getOrder().getTotal());
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setOrder(getUser().getOrder());
		payment.setProcessorClassName(this.getClass().getName());
		setPaymentVariablesToPersist(payment, parameters, responseMap);
		if (getUser().getOrder().getPayments() == null)
			getUser().getOrder().setPayments(new LinkedHashSet());
		getUser().getOrder().getPayments().add(payment);
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
	}

}
