package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import urn.ebay.api.PayPalAPI.DoDirectPaymentReq;
import urn.ebay.api.PayPalAPI.DoDirectPaymentRequestType;
import urn.ebay.api.PayPalAPI.DoDirectPaymentResponseType;
import urn.ebay.api.PayPalAPI.PayPalAPIInterfaceServiceService;
import urn.ebay.apis.CoreComponentTypes.BasicAmountType;
import urn.ebay.apis.eBLBaseComponents.AckCodeType;
import urn.ebay.apis.eBLBaseComponents.AddressType;
import urn.ebay.apis.eBLBaseComponents.CountryCodeType;
import urn.ebay.apis.eBLBaseComponents.CreditCardDetailsType;
import urn.ebay.apis.eBLBaseComponents.CreditCardTypeType;
import urn.ebay.apis.eBLBaseComponents.CurrencyCodeType;
import urn.ebay.apis.eBLBaseComponents.DoDirectPaymentRequestDetailsType;
import urn.ebay.apis.eBLBaseComponents.PayerInfoType;
import urn.ebay.apis.eBLBaseComponents.PaymentActionCodeType;
import urn.ebay.apis.eBLBaseComponents.PaymentDetailsType;
import urn.ebay.apis.eBLBaseComponents.PersonNameType;

import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * Processes a payment in the store for a given user's cart through PayPal
 * Direct. The {@link #processDoDirectPayment(Map)} method manages the process
 * of sending a request to PayPal, and handling the response appropriately.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class PayPalDirectProcessor extends PayPalExpressProcessor implements
		IPayPalDirectPaymentProcessor {

	static Log log = LogFactory.getLog(PayPalDirectProcessor.class);

	public Map processDoDirectPayment(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing DoDirectPayment.");

		// Create the service
		PayPalAPIInterfaceServiceService service = new PayPalAPIInterfaceServiceService(createPaypalConfigMap());

		DoDirectPaymentRequestType pprequest = new DoDirectPaymentRequestType();
		DoDirectPaymentRequestDetailsType details = new DoDirectPaymentRequestDetailsType();
		PaymentDetailsType paymentDetails = new PaymentDetailsType();
		paymentDetails.setButtonSource("Java_SDK_JSP");

		// Set the currency
		String currency = (String) getSettings().getValue("currencyCode");

		// Set the order total
		String orderTotalString = getUser().getOrder().getTotal().toString();
		BasicAmountType orderTotal = new BasicAmountType();
		orderTotal.setValue(orderTotalString);
		orderTotal.setCurrencyID(CurrencyCodeType.fromValue(currency));
		paymentDetails.setOrderTotal(orderTotal);
		paymentDetails.setButtonSource("SoftSlateLLC_Cart");

		// Set an optional custom field
		if (getUser().getCustomMap() != null
				&& getUser().getCustomMap().get("payPalDirectCustom") != null) {
			paymentDetails.setCustom(getUser().getCustomMap().get(
					"payPalDirectCustom").toString());
		}

		// Set the invoice id as the orderID
		paymentDetails.setInvoiceID(Integer.toString(getUser().getOrder()
				.getOrderID()));

		// Set the ship to address
		AddressType shipTo = new AddressType();
		if (getUser().getOrder().firstDelivery().getFirstName() != null
				&& getUser().getOrder().firstDelivery().getLastName() != null) {
			shipTo.setName(getUser().getOrder().firstDelivery().getFirstName()
					+ " " + getUser().getOrder().firstDelivery().getLastName());
		}
		if (getUser().getOrder().firstDelivery().getAddress1() != null) {
			shipTo.setStreet1(getUser().getOrder().firstDelivery()
					.getAddress1());
		}
		if (getUser().getOrder().firstDelivery().getAddress2() != null) {
			shipTo.setStreet2(getUser().getOrder().firstDelivery()
					.getAddress2());
		}
		if (getUser().getOrder().firstDelivery().getCity() != null) {
			shipTo.setCityName(getUser().getOrder().firstDelivery().getCity());
		}
		if (getUser().getOrder().firstDelivery().getState() != null) {
			shipTo.setStateOrProvince(getUser().getOrder().firstDelivery()
					.getState());
		}
		if (getUser().getOrder().firstDelivery().getCountry() != null) {
			shipTo.setCountry(CountryCodeType.fromValue(getUser().getOrder()
					.firstDelivery().getCountry()));
		}
		if (getUser().getOrder().firstDelivery().getPostalCode() != null) {
			shipTo.setPostalCode(getUser().getOrder().firstDelivery()
					.getPostalCode());
		}
		paymentDetails.setShipToAddress(shipTo);

		details.setPaymentDetails(paymentDetails);

		// Set credit card details
		CreditCardDetailsType cardDetails = new CreditCardDetailsType();
		String ccType = (String) parameters.get("creditCardType");
		if (ccType.equals("VISA")) {
			ccType = "Visa";
		} else if (ccType.equals("MASTERCARD")) {
			ccType = "MasterCard";
		}
		if (ccType.equals("DISCOVER")) {
			ccType = "Discover";
		}
		if (ccType.equals("AMEX")) {
			ccType = "Amex";
		}
		cardDetails.setCreditCardType(CreditCardTypeType.fromValue(ccType));
		cardDetails.setCreditCardNumber((String) parameters
				.get("decryptedNumber"));
		cardDetails.setExpMonth(Integer.parseInt((String) parameters
				.get("creditCardExpirationMonth")));
		cardDetails.setExpYear(Integer.parseInt((String) parameters
				.get("creditCardExpirationYear")));
		cardDetails.setCVV2((String) parameters.get("creditCardSecurityCode"));

		// Set billing/card owner info
		PayerInfoType payer = new PayerInfoType();
		PersonNameType name = new PersonNameType();
		name.setFirstName(getUser().getOrder().getFirstName());
		name.setLastName(getUser().getOrder().getLastName());
		payer.setPayerName(name);
		payer.setPayerCountry(CountryCodeType.fromValue(getUser().getOrder()
				.getCountry()));
		AddressType billTo = new AddressType();
		if (getUser().getOrder().getFirstName() != null
				&& getUser().getOrder().getLastName() != null) {
			billTo.setName(getUser().getOrder().getFirstName() + " "
					+ getUser().getOrder().getLastName());
		}
		if (getUser().getOrder().getAddress1() != null) {
			billTo.setStreet1(getUser().getOrder().getAddress1());
		}
		if (getUser().getOrder().getAddress2() != null) {
			billTo.setStreet2(getUser().getOrder().getAddress2());
		}
		if (getUser().getOrder().getCity() != null) {
			billTo.setCityName(getUser().getOrder().getCity());
		}
		if (getUser().getOrder().getState() != null) {
			billTo.setStateOrProvince(getUser().getOrder().getState());
		}
		if (getUser().getOrder().getCountry() != null) {
			billTo.setCountry(CountryCodeType.fromValue(getUser().getOrder()
					.getCountry()));
		}
		if (getUser().getOrder().getPostalCode() != null) {
			billTo.setPostalCode(getUser().getOrder().getPostalCode());
		}
		payer.setAddress(billTo);

		cardDetails.setCardOwner(payer);

		details.setCreditCard(cardDetails);

		// Set the customer's IP address
		String ipAddress = (String) parameters.get("ipAddress");
		if (ipAddress.startsWith("0:")) {
			ipAddress = "127.0.0.1";
		}
		details.setIPAddress(ipAddress);

		// Set the transaction type
		String paymentType = (String) getSettings().getValue(
				"paypalPaymentType");
		details.setPaymentAction(PaymentActionCodeType.fromValue(paymentType));

		pprequest.setDoDirectPaymentRequestDetails(details);
		DoDirectPaymentReq doPaymentReq = new DoDirectPaymentReq();
		doPaymentReq.setDoDirectPaymentRequest(pprequest);

		// Send the request
		DoDirectPaymentResponseType ppresponse = (DoDirectPaymentResponseType) service.doDirectPayment(doPaymentReq);

		// Save the response
		Map results = new HashMap();
		results.put("ppresponse", ppresponse);

		if (ppresponse.getAck().equals(AckCodeType.SUCCESS)
				|| ppresponse.getAck().equals(AckCodeType.SUCCESSWITHWARNING)) {

			// Process a successful response
			parameters.put("orderTotal", orderTotalString);
			parameters.put("transactionID", ppresponse.getTransactionID());
			parameters.put("avsCode", ppresponse.getAVSCode());
			parameters.put("cvv2Code", ppresponse.getCVV2Code());
			parameters.put("paymentType", paymentType);

			if (log.isDebugEnabled())
				log.debug("Result parameters: " + parameters);

			processSuccessfulTransaction(parameters);

			results.put("resultCode", "0");
		} else {

			// Non successful response - save errors and return a failure
			if (ppresponse.getErrors() != null && ppresponse.getErrors().size() > 0) {
				results.put("resultCode", ppresponse.getErrors().get(0)
						.getErrorCode());
				results.put("resultMessage", ppresponse.getErrors().get(0)
						.getLongMessage());
				if (log.isDebugEnabled())
					log.debug("Error results from paypal: " + results);
			}
		}

		return results;
	}

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");

		Map results = processDoDirectPayment(parameters);

		return results;
	}

	public void processSuccessfulTransaction(Map parameters) throws Exception {
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		BeanUtils.copyProperties(payment, parameters);
		payment.setStatus("Received");
		payment.setAmount(getUser().getOrder().getTotal());
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setOrder(getUser().getOrder());
		payment.setProcessorClassName(this.getClass().getName());
		String creditCardNumberDisplay = null;
		String decryptedNumber = payment.getDecryptedNumber();
		if (decryptedNumber != null) {
			creditCardNumberDisplay = "XXXXXXXXXXXX"
					+ decryptedNumber.substring(decryptedNumber.length() - 4,
							decryptedNumber.length());
			payment.setCreditCardNumberDisplay(creditCardNumberDisplay);
		}
		payment
				.setValueFieldLabels("Order Total,AVS Result Code,CVV2 Result Code,,Payment Type,,Transaction ID,,");
		payment.setValue1((String) parameters.get("orderTotal"));
		payment.setValue2((String) parameters.get("avsCode"));
		payment.setValue3((String) parameters.get("cvv2Code"));
		payment.setValue5((String) parameters.get("paymentType"));
		payment.setValue7((String) parameters.get("transactionID"));
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
		if (getUser().getOrder().getPayments() == null) {
			getUser().getOrder().setPayments(new LinkedHashSet());
		}
		getUser().getOrder().getPayments().add(payment);
	}
}
