package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.payment.paypal.PayPalConstants;

/**
 * Processes a payment in the store for a given user's cart. This implementation
 * processes a request through PayPal Payflow Pro. The {@link #process(Map)}
 * method manages the process of sending a request to Payflow Pro, and handling
 * the response appropriately. It follows these steps:
 * <ol>
 * <li>Figures out the transaction type (sale, authorize,
 * authorize/authenticate, or authorize/authenticate/sale) from the database
 * settings and the user's current state.</li>
 * <li>Builds the request to Payflow Pro as a <code>Map</code>, taking as
 * input the current user's state, and various database settings.</li>
 * <li>Converts the request into a url-encoded query string to pass as Payflow
 * Pro's "parmlist" parameter.</li>
 * <li>Initializes the connection to Payflow Pro and sends the request.</li>
 * <li>Handles any errors by setting "resultCode" appropriately in the
 * <code>Map</code> that is returned. If no response comes back from Payflow
 * Pro, a result code 10000 is used. If the Payflow Pro result code is not 0,
 * which also indicates an error, that code is used as this method's
 * "resultCode" and this method returns it.</li>
 * <li>If a successful response is received, checks if the transaction type for
 * the user is AA or AAS (authorize/authenticate, or
 * authorize/authenticate/sale), in which the response must be "authenticated".
 * This means the AVS and CSC responses are checked against what's defined as
 * acceptable in the Payflow Pro Advanced Settings in the administrator. Note
 * that if no CSC response is present, it is set to "X".</li>
 * <li>If the user is not deemed authenticated, a "resultCode" of 10001 is
 * returned.</li>
 * <li>If the user is authenticated and the transaction type in effect is AAS
 * (authorize/authenticate/sale), an immediate delayed capture on the previous
 * authorization is performed, using the same parameters. Any errors from this
 * second request are handled in the same way as above.</li>
 * <li>Assuming the transaction succeeds, a new record is written to the
 * sscPayment database table and a resultCode of 0 is returned.</li>
 * </ol>
 * 
 * @author David Tobey
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class PayflowProProcessor extends BasePayPalNVPProcessor implements
		PaymentProcessor {

	static Log log = LogFactory.getLog(PayflowProProcessor.class);

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");
		Map result = new HashMap();

		// Figure out the transaction type
		String transAuthLevel = getTransAuthLevel();
		if (log.isDebugEnabled())
			log.debug("transAuthLevel: " + transAuthLevel);
		String transactionType = (String) getSettings().getValue(
				"payflowProType" + transAuthLevel);
		String trxtype = transactionType;
		if (transactionType.equals(PayPalConstants.AUTH_AUTHENTICATE_TRXTYPE)
				|| transactionType
						.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			trxtype = PayPalConstants.AUTH_TRXTYPE;
		}
		parameters.put(PayPalConstants.TRXTYPE, trxtype);
		parameters.put(PayPalConstants.TENDER,
				PayPalConstants.DIRECT_PAYMENT_TENDER);
		String response = this.makePayPalCall(parameters, "payflowPro");
		if (log.isDebugEnabled())
			log.debug("response: " + response);

		// Parse the response into a Map
		Map responseMap = parseResponseData(response);

		// Handle the response - errors first
		String resultCode = (String) responseMap.get(PayPalConstants.RESULT);
		if (resultCode == null) {
			result.put("resultCode", "10000");
			return result;
		} else if (!resultCode.equals("0")) {
			result.put("resultCode", resultCode);
			result.put("resultMessage", (String) responseMap
					.get(PayPalConstants.RESPMSG));
			return result;
		}

		// Successful response - if necessary, authenticate
		boolean authenticated = true;
		if (transactionType.equals(PayPalConstants.AUTH_AUTHENTICATE_TRXTYPE)
				|| transactionType
						.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			authenticated = isAuthenticated(transAuthLevel, responseMap);
		}

		// Send back error if can't authenticate
		if (!authenticated) {
			String avsAddr = (String) responseMap
					.get(PayPalConstants.AVS_ADDR_RESP);
			String avsZip = (String) responseMap
					.get(PayPalConstants.AVS_ZIP_RESP);
			String cvv2 = (String) responseMap
					.get(PayPalConstants.AVS_CVV2_RESP);
			if (cvv2 == null) {
				cvv2 = "X";
			}
			result.put("resultCode", "10001");
			if ((!avsAddr.equals("Y") || !avsZip.equals("Y"))
					&& !cvv2.equals("Y"))
				result.put("resultCode", "10001");
			else if ((!avsAddr.equals("Y") || !avsZip.equals("Y"))
					&& cvv2.equals("Y"))
				result.put("resultCode", "10002");
			else if ((avsAddr.equals("Y") && avsZip.equals("Y"))
					&& !cvv2.equals("Y"))
				result.put("resultCode", "10003");
			return result;
		}

		// If transaction type is AAS (Authorize/Authenticate/Sale), run an
		// immediate delayed capture
		if (transactionType
				.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {

			// Update param map to use previous PNREF in a delayed capture
			parameters.put(PayPalConstants.PNREF, (String) responseMap
					.get(PayPalConstants.PNREF));
			parameters.put(PayPalConstants.TRXTYPE,
					PayPalConstants.CAPTURE_TRXTYPE);
			response = this.makePayPalCall(parameters, "payflowPro");
			if (log.isDebugEnabled())
				log.debug("response: " + response);

			// Parse the response into a Map
			responseMap = parseResponseData(response);

			// Handle any response errors
			resultCode = (String) responseMap.get(PayPalConstants.RESULT);
			if (resultCode == null) {
				result.put("resultCode", "10000");
				return result;
			} else if (!resultCode.equals("0")) {
				result.put("resultCode", resultCode);
				result.put("resultMessage", (String) responseMap
						.get(PayPalConstants.RESPMSG));
				return result;
			}
		}

		// Successful transaction - write it to the database
		responseMap.put("transactionType", transactionType);
		processSuccessfulTransaction(parameters, responseMap);

		result.put("resultCode", "0");
		return result;
	}

	public String getTransAuthLevel() throws Exception {
		String cust1 = (String) getSettings().getValue("payflowProCust1");
		String cust2 = (String) getSettings().getValue("payflowProCust2");
		String cust3 = (String) getSettings().getValue("payflowProCust3");
		String floor1 = (String) getSettings().getValue("payflowProFloor1");
		String floor2 = (String) getSettings().getValue("payflowProFloor2");
		String floor3 = (String) getSettings().getValue("payflowProFloor3");
		String ceiling1 = (String) getSettings().getValue("payflowProCeiling1");
		String ceiling2 = (String) getSettings().getValue("payflowProCeiling2");
		String ceiling3 = (String) getSettings().getValue("payflowProCeiling3");
		String countries1 = (String) getSettings().getValue(
				"payflowProCountries1");
		String countries2 = (String) getSettings().getValue(
				"payflowProCountries2");
		String countries3 = (String) getSettings().getValue(
				"payflowProCountries3");
		if (levelMatchFound(cust1, floor1, ceiling1, countries1)) {
			return "1";
		} else if (levelMatchFound(cust2, floor2, ceiling2, countries2)) {
			return "2";
		} else if (levelMatchFound(cust3, floor3, ceiling3, countries3)) {
			return "3";
		}
		return "";
	}

	public boolean levelMatchFound(String cust, String floor, String ceiling,
			String countries) {
		if (getUser().getCustomer() != null
				&& getUser().getCustomer().getStatus() != null
				&& getUser().getCustomer().getStatus().equals(cust)) {
			return true;
		}
		try {
			double floorDouble = Double.parseDouble(floor);
			double ceilingDouble = Double.parseDouble(ceiling);
			if (getUser().getOrder().getTotal().doubleValue() > floorDouble
					&& (getUser().getOrder().getTotal().doubleValue() < ceilingDouble || ceilingDouble == 0)) {
				return true;
			}
		} catch (NumberFormatException e) {
			log.debug("Number Parsing Error", e);
		}
		if (getUser().getOrder().firstDelivery().getCountry() != null
				&& countries.indexOf(getUser().getOrder().firstDelivery()
						.getCountry()) != -1) {
			return true;
		}
		return false;
	}

	protected Map preparePaypalRequestAcutalPayload(Map params, Map parameters) {

		// Prepare the request to PayPal Payflow Pro
		if (null == params)
			params = new HashMap();
		String firstname = "";
		String lastname = "";
		String address = "";
		String city = "";
		String companyname = "";
		String state = "";
		String zip = "";
		String country = "";
		String email = "";
		String shiptofirstname = "";
		String shiptolastname = "";
		String nameToShip = "";
		String addressToShip = "";
		String cityToShip = "";
		String stateToShip = "";
		String zipToShip = "";
		if (getUser().getOrder().getFirstName() != null)
			firstname = getUser().getOrder().getFirstName();
		if (getUser().getOrder().getLastName() != null)
			lastname = getUser().getOrder().getLastName();
		if (getUser().getOrder().getAddress1() != null)
			address = getUser().getOrder().getAddress1();
		if (getUser().getOrder().getAddress2() != null)
			address = address + " " + getUser().getOrder().getAddress2();
		if (getUser().getOrder().getCity() != null)
			city = getUser().getOrder().getCity();
		if (getUser().getOrder().getOrganization() != null)
			companyname = getUser().getOrder().getOrganization();
		if (getUser().getOrder().getState() != null)
			state = getUser().getOrder().getState();
		if (getUser().getOrder().getPostalCode() != null)
			zip = getUser().getOrder().getPostalCode();
		if (getUser().getOrder().getCountry() != null)
			country = getUser().getOrder().getCountry();
		if (getUser().getOrder().getEmail1() != null)
			email = getUser().getOrder().getEmail1();
		if (getUser().getOrder().firstDelivery().getFirstName() != null)
			shiptofirstname = getUser().getOrder().firstDelivery()
					.getFirstName();
		if (getUser().getOrder().firstDelivery().getLastName() != null)
			shiptolastname = getUser().getOrder().firstDelivery().getLastName();
		if (getUser().getOrder().firstDelivery().getFirstName() != null)
			nameToShip = getUser().getOrder().firstDelivery().getFirstName();
		if (getUser().getOrder().firstDelivery().getLastName() != null)
			nameToShip = nameToShip + " "
					+ getUser().getOrder().firstDelivery().getLastName();
		if (getUser().getOrder().firstDelivery().getAddress1() != null)
			addressToShip = getUser().getOrder().firstDelivery().getAddress1();
		if (getUser().getOrder().firstDelivery().getAddress2() != null)
			addressToShip = addressToShip + " "
					+ getUser().getOrder().firstDelivery().getAddress2();
		if (getUser().getOrder().firstDelivery().getCity() != null)
			cityToShip = getUser().getOrder().firstDelivery().getCity();
		if (getUser().getOrder().firstDelivery().getState() != null)
			stateToShip = getUser().getOrder().firstDelivery().getState();
		if (getUser().getOrder().firstDelivery().getPostalCode() != null)
			zipToShip = getUser().getOrder().firstDelivery().getPostalCode();
		params.put(PayPalConstants.TAXAMT, getUser().getOrder().getTax()
				.toString());
		params.put(PayPalConstants.FREIGHTAMT, getUser().getOrder()
				.getShipping().toString());
		params.put(PayPalConstants.NAME, (String) parameters
				.get("creditCardName"));
		params.put(PayPalConstants.ACCT, (String) parameters
				.get("decryptedNumber"));
		params.put(PayPalConstants.CVV2, (String) parameters
				.get("creditCardSecurityCode"));
		String creditCardExpirationYear = (String) parameters
				.get("creditCardExpirationYear");
		params.put(PayPalConstants.EXPDATE, (String) parameters
				.get("creditCardExpirationMonth")
				+ creditCardExpirationYear.substring(2, 4));
		params.put(PayPalConstants.FIRSTNAME, firstname);
		params.put(PayPalConstants.LASTNAME, lastname);
		params.put(PayPalConstants.COMPANYNAME, companyname);
		params.put(PayPalConstants.STREET, address);
		params.put(PayPalConstants.CITY, city);
		params.put(PayPalConstants.STATE, state);
		params.put(PayPalConstants.ZIP, zip);
		params.put(PayPalConstants.COUNTRY, country);
		params.put(PayPalConstants.EMAIL, email);
		params.put(PayPalConstants.SHIPTOFIRSTNAME, shiptofirstname);
		params.put(PayPalConstants.SHIPTOLASTNAME, shiptolastname);
		params.put(PayPalConstants.SHIPTOSTREET, addressToShip);
		params.put(PayPalConstants.SHIPTOCITY, cityToShip);
		params.put(PayPalConstants.SHIPTOSTATE, stateToShip);
		params.put(PayPalConstants.SHIPTOZIP, zipToShip);
		params.put(PayPalConstants.CUSTREF, (String) parameters.get("custRef"));
		params.put(PayPalConstants.CUSTCODE, (String) parameters
				.get("customerCode"));
		return params;
	}

	public boolean isAuthenticated(String transAuthLevel, Map responseMap)
			throws Exception {
		String cvv2 = (String) responseMap.get(PayPalConstants.AVS_CVV2_RESP);
		if (cvv2 == null) {
			cvv2 = "X";
		}
		String streetZipCsc = (String) responseMap
				.get(PayPalConstants.AVS_ADDR_RESP)
				+ (String) responseMap.get(PayPalConstants.AVS_ZIP_RESP) + cvv2;
		String authMatches = (String) getSettings().getValue(
				"payflowProAuth" + transAuthLevel);
		if (authMatches.indexOf(streetZipCsc) != -1) {
			return true;
		}
		return false;
	}

	protected void setPaymentVariablesToPersist(Payment payment,
			Map parameters, Map responseMap) {
		String creditCardNumberDisplay = null;
		String decryptedNumber = payment.getDecryptedNumber();
		if (decryptedNumber != null) {
			creditCardNumberDisplay = "XXXXXXXXXXXX"
					+ decryptedNumber.substring(decryptedNumber.length() - 4,
							decryptedNumber.length());
			payment.setCreditCardNumberDisplay(creditCardNumberDisplay);
		}
		payment
				.setValueFieldLabels("AVSADDR,AVSZIP,CVV2MATCH,RESULT, TRXTYPE,AUTHCODE,PNREF,RESPMSG,");
		payment.setValue1((String) responseMap
				.get(PayPalConstants.AVS_ADDR_RESP));
		payment.setValue2((String) responseMap
				.get(PayPalConstants.AVS_ZIP_RESP));
		payment.setValue3((String) responseMap
				.get(PayPalConstants.AVS_CVV2_RESP));
		payment.setValue4((String) responseMap.get(PayPalConstants.RESULT));
		payment.setValue5((String) responseMap.get("transactionType"));
		payment.setValue6((String) responseMap
				.get(PayPalConstants.AUTH_CODE_RESP));
		payment.setValue7((String) responseMap.get(PayPalConstants.PNREF));
		payment.setValue8((String) responseMap.get(PayPalConstants.RESPMSG));
	}
}
