package com.softslate.commerce.customer.core;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;

import com.softslate.commerce.daos.core.AppRevisionListener;
import com.softslate.commerce.daos.core.HibernateDAOFactory;

/**
 * A servlet filter that manages a single Hibernate Session for each request,
 * per the ThreadLocal model. (See www.hibernate.org for more information.)
 * <p>
 * This filter commits the thread's transaction at the end of the request, if it
 * is still open. Then it closes the Hibernate <code>Session</code>.
 * 
 * @author David Tobey
 */
public class HibernateFilter extends BaseFilter implements Filter {

	private static Log log = LogFactory.getLog(HibernateFilter.class);

	private FilterConfig filterConfig = null;

	public void init(FilterConfig filterConfig) throws ServletException {
		this.filterConfig = filterConfig;
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		// Must cast incoming request to HttpServletRequest
		HttpServletRequest req = (HttpServletRequest) request;

		// Ignore the request if it is an image.
		if (isStaticRequest(req)) {
			chain.doFilter(request, response);
			return;
		}

		if (log.isDebugEnabled())
			log.debug("HibernateFilter in effect.");

		// Find the HibernateDAOFactory in the application scope
		HibernateDAOFactory daoFactory = (HibernateDAOFactory) filterConfig
				.getServletContext().getAttribute("daoFactory");

		// Close the session in case it was not closed in the previous thread it
		// was attached to
		if (daoFactory != null) {
			if (log.isInfoEnabled())
				log.info("Checking if previous session wasn't closed.");
			daoFactory.closeSession();
		}

		try {

			// Do nothing before the request
			chain.doFilter(request, response);

		} finally {

			// Commit the request's transaction if it is still open.
			if (log.isDebugEnabled())
				log.debug("Cleaning up Hibernate transaction and session.");
			if (daoFactory != null) {
				if (log.isDebugEnabled())
					log.debug("Committing Hibernate transaction (if still open).");
				try {
					daoFactory.commitTransaction();
				} catch (HibernateException he) {
					// Swallow here. It's logged within the DAOFactory
				}
			}

			// Always close the Hibernate Session at the end of the request.
			if (daoFactory != null) {
				if (log.isDebugEnabled())
					log.debug("Closing Hibernate session (if open).");
				daoFactory.closeSession();
			}

			// Clear the thread's request log for Enver's AppRevisionListener
			AppRevisionListener.setRequestLogForThread(null);
		}
	}

	public void destroy() {
	}

}
