package com.softslate.commerce.customer.product;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.ProductReview;
import com.softslate.commerce.businessobjects.product.ProductReviewProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * This action processes the review submitted by the user.
 * 
 * @author Tony Zakula
 */
public class ProductReviewAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductReviewAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		ProductReviewForm baseForm = (ProductReviewForm) form;
		baseForm.initializeProperties(mapping, request);

		synchronized (AppUtils.getSessionLock(request.getSession())) {

			// If the user is not logged in, it's an error and security check.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.userNotLoggedIn"));
				return mapping.findForward("notLoggedIn");
			}
			String enableProductReviews = (String) baseForm.getSettings().getValue("enableProductReviews");
			if (enableProductReviews == null || enableProductReviews.equals("0")) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.productReviewsNotEnabled"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Get the product data from the database
			Product product = loadProduct(baseForm);

			// If that fails, it's an error
			if (product.getProductID() == 0) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.productNotFound", product.getCode()));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Check if submitting more than one review per product by the same
			// customer is allowed.
			String allowCustomersToSubmitMultipleReviewsPerProduct = (String) baseForm.getSettings().getValue(
					"allowCustomersToSubmitMultipleReviewsPerProduct");
			if (allowCustomersToSubmitMultipleReviewsPerProduct != null
					&& allowCustomersToSubmitMultipleReviewsPerProduct.equals("0")) {
				boolean customerSubmittedReview = customerSubmittedReviewForProduct(baseForm.getUser().getCustomer(),
						product);
				if (customerSubmittedReview) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.onlyOneReviewAllowedPerProduct"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					return mapping.findForward("failure");
				}
			}

			// Try uploading any files, these will throw exceptions if they do
			// not work
			uploadFiles(baseForm);

			// Set initial state to unapproved
			Map parameters = PropertyUtils.describe(baseForm);
			parameters.put("status", new Integer(0));
			parameters.put("customerID", new Integer(baseForm.getUser().getCustomer().getCustomerID()));
			parameters.put("ipAddress", request.getRemoteAddr());
			parameters.put("email", baseForm.getUser().getCustomer().getEmail1());
			parameters.put("productID", new Integer(product.getProductID()));

			// Now insert record into database.
			ProductReviewProcessor prp = (ProductReviewProcessor) baseForm.getBusinessObjectFactory().createObject(
					"productReviewProcessorImplementer");
			Map results = prp.addProductReview(parameters);
			if (results.get("newRecord") != null) {
				ProductReview productReview = (ProductReview) results.get("newRecord");
				request.setAttribute("productReview", productReview);
				request.setAttribute("product", product);

				// Success message
				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.reviewAdded"));
				saveMessages(baseForm.getRequest(), baseForm.getMessages());

				// Process the email
				sendAdministratorEmail(baseForm, productReview);
			}
			return mapping.findForward("success");
		}
	}

	protected Product loadProduct(ProductReviewForm baseForm) throws Exception {
		ProductProcessor productProcessor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
				"productProcessorImplementer");
		Product product =  baseForm.getBusinessObjectFactory().createObject(Product.class);
		product.setCode(baseForm.getCode());
		product = productProcessor.getProductFromCode(product);
		return product;
	}

	protected void uploadFiles(ProductReviewForm baseForm) throws Exception {
		ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
				"actionUtilsImplementer");

		// Load the imageSystemPath and imagePathURL settings; derive the
		// imageSystemPath from the URL path if blank.
		String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
		String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
		if (!imagePathURL.endsWith("/")) {
			imagePathURL = imagePathURL + "/";
		}
		if (imageSystemPath == null || imageSystemPath.length() == 0) {
			imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
					+ System.getProperty("file.separator");
			if (!imagePathURL.startsWith("/")) {
				imagePathURL = "/" + imagePathURL;
			}
			imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;
		}

		// Now process any uploaded images
		if (baseForm.getImageFile() != null && baseForm.getImageFile().getFileSize() > 0) {
			// generate random number to not have dupe images
			String filename = UUID.randomUUID().toString() + "." + getFileExt(baseForm.getImageFile().getFileName());
			actionUtils.saveFile(imageSystemPath + filename, baseForm.getImageFile().getInputStream());
			baseForm.setImagePath(imagePathURL + filename);
		}
	}

	protected boolean customerSubmittedReviewForProduct(Customer customer, Product product) {
		boolean ret = false;
		if (product.getProductReviews() != null && customer != null) {
			Iterator i = product.getProductReviews().iterator();
			while (i.hasNext()) {
				ProductReview pr = (ProductReview) i.next();
				if (pr.getCustomer() != null && pr.getCustomer().getCustomerID() == customer.getCustomerID()) {
					ret = true;
				}
			}
		}
		return ret;
	}

	public void sendAdministratorEmail(ProductReviewForm baseForm, ProductReview productReview) {
		try {
			ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
					"actionUtilsImplementer");
			// Get email parameters from the settings
			String toEmails = (String) baseForm.getSettings().getValue("productReviewEmailTo");
			String ccEmails = "";
			String bccEmails = "";
			String subject = (String) baseForm.getSettings().getValue("productReviewEmailSubject");
			String fromEmail = (String) baseForm.getSettings().getValue("productReviewEmailFrom");
			if (fromEmail == null || fromEmail.length() == 0) {
				fromEmail = (String) baseForm.getSettings().getValue("storeEmailAddress");
			}
			String smtpServer = (String) baseForm.getSettings().getValue("storeSMTPServer");
			String smtpUsername = (String) baseForm.getSettings().getValue("storeSMTPUsername");
			String smtpPassword = (String) baseForm.getSettings().getValue("storeSMTPPassword");
			String smtpPort = (String) baseForm.getSettings().getValue("storeSMTPPort");
			String characterEncoding = (String) baseForm.getSettings().getValue("characterEncoding");
			String adminURL = (String) baseForm.getSettings().getValue("administratorURL");
			adminURL = adminURL + "/ProductReviewEditForm.do?productReviewID=" + productReview.getProductReviewID();
			String textMessage = constructEmailText(baseForm.getRequest(), adminURL);
			String htmlMessage = constructEmailHTML(baseForm.getRequest(), adminURL);

			// Send an email to let us know
			actionUtils.sendMultipartEmail(toEmails.trim(), ccEmails.trim(), bccEmails.trim(), fromEmail.trim(),
					smtpServer, subject, textMessage, htmlMessage, smtpUsername.trim(), smtpPassword.trim(),
					smtpPort.trim(), characterEncoding.trim());
		} catch (Exception e) {
			log.warn("Error sending contact email:", e);
		}

	}

	public String constructEmailText(HttpServletRequest request, String adminURL) {
		Enumeration keys = request.getParameterNames();
		StringBuffer sb = new StringBuffer();
		sb.append("Hello,\r\n\r\nA new product review has been submitted by a customer. "
				+ "It is inactive now, awaiting your approval. Here are the details:\r\n\r\n");
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			sb.append(key);
			sb.append(" - ");
			sb.append(request.getParameter(key));
			sb.append("\r\n");
		}
		sb.append("\r\n");
		sb.append("View Review Details and Approve This Review");
		sb.append(adminURL);
		sb.append("\r\n");
		return sb.toString();
	}

	public String constructEmailHTML(HttpServletRequest request, String adminURL) {
		Enumeration keys = request.getParameterNames();
		StringBuffer sb = new StringBuffer();
		sb.append("<html><body><p>");
		sb.append("Hello,<br/><br/>A new product review has been submitted by a customer."
				+ "It is inactive now, awaiting your approval. Here are the details:<br /><br />");
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			sb.append(key);
			sb.append(" - ");
			sb.append(request.getParameter(key));
			sb.append("<br />");
		}
		sb.append("</p><p>");
		sb.append("<a href=");
		sb.append(adminURL);
		sb.append(">");
		sb.append("View Review Details and Approve This Review");
		sb.append("</a>");
		sb.append("</p></body></html>");
		return sb.toString();
	}

	private String getFileExt(String fmtFileName) {
		int dotPos = fmtFileName.lastIndexOf(".");
		return fmtFileName.substring(dotPos + 1);
	}

}
